# -*- coding: utf8 -*-

import cPickle
import os

RONDA = {
    '2': "2° ronda",
    '3': "3° ronda",
    '4': "4° ronda",
    'O': "octavos",
    'C': "cuartos",
    'S': "semis",
    'F': "la final",
}

TTYPES = {
    'ms': "hombres, singles",
    'md': "hombres, dobles",
}

class Informer(object):
    def __init__(self, dbfilename=None):
        if dbfilename is None:
            dbfilename = 'informer.db'
        self._fname = dbfilename
        if os.path.exists(dbfilename):
            with open(dbfilename) as fh:
                self.db = cPickle.load(fh)
        else:
            self.db = {}

    def report(self, tour, ttype, player_info):
        """Report."""
        # use just last names
        processed_info = []
        for name, info in player_info:
            processed_info.append((name, info))

        # make the report
        try:
            tour_data = self.db[(tour, ttype)]
        except KeyError:
            # first pass
            tour_data, resp = self._rep_init(processed_info)
        else:
            # rest of tournament
            data = self._rep_others(processed_info, tour_data)
            if data:
                tour_data, resp = data

        self.db[(tour, ttype)] = tour_data
        self.save()

        text = u"%s (%s): %s" % (tour, TTYPES[ttype], u"".join(resp))
        return text

    def _rep_init(self, player_info):
        """Make the report in case of a first call."""
        text = [u"Arranca el torneo"]
        if len(player_info) == 0:
            text.append(u" sin argentinos.")
            tour_data = {}

        elif len(player_info) == 1:
            name, seq = player_info[0]
            text.append(u", juega %s" % (name))
            if seq[0] == 'P':
                text.append(u" preclasificado")
            tour_data = {name, seq[0]}

        else:
            tour_data = dict((n, s[0]) for n, s in player_info)
            # need to separate preclassified from the others
            preclas = [n for n, s in player_info if s[0] == 'P']
            others = [n for n, s in player_info if s[0] != 'P']
            text.append(u", juegan ")

            # first, preclassifieds
            if preclas:
                if len(preclas) == 1:
                    text.append(u"%s preclasificado" % (preclas[0],))
                else:
                    primeros = preclas[:-1]
                    ultimo = preclas[-1]
                    text.append(u"%s y %s preclasificados" % (
                                                 u', '.join(primeros), ultimo))

            # others
            if others:
                if preclas:
                    text.append(u", y ")
                if len(others) == 1:
                    text.append(others[0])
                else:
                    primeros = others[:-1]
                    ultimo = others[-1]
                    text.append(u"%s y %s" % (u', '.join(primeros), ultimo))

        return tour_data, text

    def  _rep_others(self, player_info, tour_data):
        """Make the report for the other calls."""
        if not tour_data:
            return ''

        happened = []

        for name, new_seq in player_info:
            prev_seq = tour_data[name]
            if new_seq == prev_seq:
                # nothing new
                continue

            new_pos = len(prev_seq)
            new_item = new_seq[new_pos]
            if new_item == '?':
                # don't care, it's not a result
                continue

            if new_item == '|':
               happened.append(u"%s perdió" % (name,))
            elif new_item == 'W':
               happened.append(u"%s ganó el torneo!" % (name,))
            else:
               happened.append(u"%s pasó a %s" % (name, RONDA[new_item]))


            tour_data[name] = new_seq[:new_pos + 1]

        if len(happened) == 0:
            return ''
        elif len(happened) == 1:
            text = happened[0]
        else:
            prims = happened[:-1]
            ultim = happened[-1]
            text = u"%s y %s" % (', '.join(prims), ultim)

        return tour_data, text

    def save(self):
        with open(self._fname, 'wb') as fh:
            cPickle.dump(self.db, fh)


# text in function of the quantity of players in the round
# that the player just won
ADDENDUM = {
    2: u"ganando el torneo!",
    4: u"pasando a la final",
    8: u"pasando a la semifinal",
    16: u"pasando a cuartos",
    32: u"pasando a octavos",
}

# text in function of won/lost and singles/doubles
ACTION = {
    (True, True): u"le ganó a",
    (False, True): u"perdió con",
    (True, False): u"les ganaron a",
    (False, False): u"perdieron con",
}

def format_live_news(tourname, roundbase, roundwin, p1, p2, won):
    """Format the news for the live feed."""
    # what happened
    action = ACTION[(won, p1.singles)]

    # score string, without the non played sets
    scores = []
    for p1sc, p2sc, p1tb, p2tb in zip(p1.scores, p2.scores,
                                      p1.tbreaks, p2.tbreaks):
        if p1sc == 0 and p2sc == 0:
            # not useful column
            continue

        if p1tb == 0 or p2tb == 0:
            # tiebreak not played
            tbreak = ""
        else:
            tbreak = "(%d)" % (min(p1tb, p2tb),)

        p1sc, p2sc = str(p1sc), str(p2sc)
        sep = '-' if len(p1sc) > 1 or len(p2sc) > 1 else ''
        scores.append(''.join((p1sc, sep, p2sc, tbreak)))
    score_string = " ".join(scores)

    # to where the player went
    if won:
        try:
            addendum = ADDENDUM[roundwin]
        except KeyError:
            # round name, in function of the base
            rnd = roundbase // roundwin + 1
            addendum = u"pasando a %d° ronda" % (rnd,)
        addendum = u", " + addendum
    else:
        addendum = ""

    text = u"%s: %s %s %s por %s%s" % (tourname, p1.name, action, p2.name,
                                       score_string, addendum)
    return text
